#version 330 core
#auto_defines

uniform sampler2D   Frame;
uniform vec4        Param; // (WinSizeX, WinSizeY, ScaleIn, LensShift/2)
uniform vec4        Scale; // (Scale.x, Scale.y, ScaleIn.x, ScaleIn.y)
uniform vec4        Distortion;
uniform vec4        ChromAb;
uniform vec4        BackColor;

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;

out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

in  vec2  TexCoord;

layout(location = 0) out vec4  OutColor;

void main()
{
    vec4 rSq;
    vec2 lensCenter, theta, thetaDist;
    vec2 texCoord = TexCoord;
    texCoord.x *= 2.0; 

    if (texCoord.x > 1.0)
    {
        // Right eye
        texCoord.x -= 1.0;
        lensCenter = vec2(0.5 - Param.w, 0.5);
        theta = (texCoord - lensCenter) * Scale.zw;

		rSq.x = 1.0;
        rSq.y = dot(theta, theta);
        rSq.z = rSq.y * rSq.y;
        rSq.w = rSq.z * rSq.y;
        thetaDist = theta * dot(rSq, Distortion);

		// Detect whether blue texture coordinates are out of range since these will scaled out the furthest.
		texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.zw);
        if (any(bvec2(clamp(texCoord, 0.0, 1.0) - texCoord)))
            OutColor = BackColor;
        else
        {
			// Do blue texture lookup
            texCoord.x = texCoord.x * 0.5 + 0.5;
 			OutColor.b = texture(Frame, texCoord).b;

			// Do green texture lookup (no scaling)
			texCoord = lensCenter + Scale.xy * thetaDist;
            texCoord.x = texCoord.x * 0.5 + 0.5;
 			OutColor.g = texture(Frame, texCoord).g;

			// Do red texture lookup
			texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.xy);
            texCoord.x = texCoord.x * 0.5 + 0.5;
 			OutColor.r = texture(Frame, texCoord).r;

			OutColor.a = 1.0;
        }
    }
    else
    {
        // Left eye
        lensCenter = vec2(0.5 + Param.w, 0.5);
        theta = (texCoord - lensCenter) * Scale.zw;

		rSq.x = 1.0;
        rSq.y = dot(theta, theta);
        rSq.z = rSq.y * rSq.y;
        rSq.w = rSq.z * rSq.y;
        thetaDist = theta * dot(rSq, Distortion);

		// Detect whether blue texture coordinates are out of range since these will scaled out the furthest.
		texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.zw);
        if (any(bvec2(clamp(texCoord, 0.0, 1.0) - texCoord)))
            OutColor = BackColor;
        else
        {
			// Do blue texture lookup
            texCoord.x = texCoord.x * 0.5;
 			OutColor.b = texture(Frame, texCoord).b;

			// Do green texture lookup (no scaling)
			texCoord = lensCenter + Scale.xy * thetaDist;
            texCoord.x = texCoord.x * 0.5;
 			OutColor.g = texture(Frame, texCoord).g;

			// Do red texture lookup
			texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.xy);
            texCoord.x = texCoord.x * 0.5;
 			OutColor.r = texture(Frame, texCoord).r;

			OutColor.a = 1.0;
        }
    }
}

#endif
