#version 330 core
#auto_defines

uniform sampler2D   guiTexture;
uniform vec4        Param;  // (WinSizeX,  WinSizeY,  ScaleIn,   LensShift/2)
uniform vec4        Scale;  // (Scale.x,   Scale.y,   ScaleIn.x, ScaleIn.y)
uniform vec4        Scale2; // (guiSize.x, guiSize.y, guiDepth,  guiCurv)
uniform vec4        ChromAb;
uniform vec4        Distortion;

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;

out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

in  vec2  TexCoord;

layout(location = 0) out vec4  OutColor;

void main()
{
	float alpha = 0.0;
    vec4  rSq;
    vec2  lensCenter, theta, thetaDist;
    vec2  texCoord = TexCoord;
    texCoord.x *= 2.0;

    if (texCoord.x > 1.0)
    {
        // Right eye

		texCoord.x -= 1.0;
        lensCenter = vec2(0.5 - Param.w, 0.5);
        theta = (texCoord - lensCenter) * Scale.zw * Scale2.xy;

		rSq.x = 1.0;
        rSq.y = dot(theta, theta);
        rSq.z = rSq.y * rSq.y;
        rSq.w = rSq.z * rSq.y;

        thetaDist = theta * dot(rSq, Distortion);

		// Do blue texture lookup
		texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.zw);
        texCoord.x += Scale2.z;
		texCoord = (texCoord - vec2(0.5)) * Scale2.xy + vec2(0.5);

 		OutColor.ba = texture(guiTexture, texCoord).ba;
		alpha = OutColor.a;

		// Do green texture lookup (no scaling)
		texCoord = lensCenter + Scale.xy * thetaDist;
        texCoord.x += Scale2.z;
		texCoord = (texCoord - vec2(0.5)) * Scale2.xy + vec2(0.5);

 		OutColor.ga = texture(guiTexture, texCoord).ga;
		alpha += OutColor.a;

		// Do red texture lookup
		texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.xy);
        texCoord.x += Scale2.z;
		texCoord = (texCoord - vec2(0.5)) * Scale2.xy + vec2(0.5);

 		OutColor.ra = texture(guiTexture, texCoord).ra;
		alpha += OutColor.a;
		OutColor.a = alpha * 0.3333;
    }
    else
    {
        // Left eye
        lensCenter = vec2(0.5 + Param.w, 0.5);
        theta = (texCoord - lensCenter) * Scale.zw * Scale2.xy;

		rSq.x = 1.0;
        rSq.y = dot(theta, theta);
        rSq.z = rSq.y * rSq.y;
        rSq.w = rSq.z * rSq.y;

        thetaDist = theta * dot(rSq, Distortion);

		// Do blue texture lookup
		texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.zw);
        texCoord.x -= Scale2.z;
		texCoord = (texCoord - vec2(0.5)) * Scale2.xy + vec2(0.5);

 		OutColor.ba = texture(guiTexture, texCoord).ba;
		alpha = OutColor.a;

		// Do green texture lookup (no scaling)
		texCoord = lensCenter + Scale.xy * thetaDist;
        texCoord.x -= Scale2.z;
		texCoord = (texCoord - vec2(0.5)) * Scale2.xy + vec2(0.5);

 		OutColor.ga = texture(guiTexture, texCoord).ga;
		alpha += OutColor.a;

		// Do red texture lookup
		texCoord = lensCenter + Scale.xy * thetaDist * dot(rSq.xy, ChromAb.xy);
        texCoord.x -= Scale2.z;
		texCoord = (texCoord - vec2(0.5)) * Scale2.xy + vec2(0.5);

 		OutColor.ra = texture(guiTexture, texCoord).ra;
		alpha += OutColor.a;
		OutColor.a = alpha * 0.3333;
    }
}

#endif
