#version 330 core
#auto_defines

uniform sampler2D Frame;
uniform vec2    EyeToSourceUVScale;
uniform vec2    EyeToSourceUVOffset;
uniform mat4x4  EyeRotationStart;
uniform mat4x4  EyeRotationEnd;

#ifdef _VERTEX_

layout(location = 0) in  vec4  PosTimeVign;
layout(location = 1) in  vec2  TexCoord0;
layout(location = 2) in  vec2  TexCoord1;
layout(location = 3) in  vec2  TexCoord2;

out vec4  fPosition;
out vec2  fTexCoord0;
out vec2  fTexCoord1;
out vec2  fTexCoord2;
out float fVignette;

vec2    TimewarpTexCoord(vec2 TexCoord, mat4x4 rotMat)
{
    // Vertex inputs are in TanEyeAngle space for the R,G,B channels (i.e. after chromatic 
    // aberration and distortion). These are now "real world" vectors in direction (x,y,1) 
    // relative to the eye of the HMD.	Apply the 3x3 timewarp rotation to these vectors.
    vec3 transformed = (rotMat * vec4(TexCoord, 1, 1)).xyz;
    // Project them back onto the Z=1 plane of the rendered images.
    vec2 flattened = transformed.xy / transformed.z;
    // Scale them into ([0,0.5],[0,1]) or ([0.5,0],[0,1]) UV lookup space (depending on eye)
    return EyeToSourceUVScale * flattened + EyeToSourceUVOffset;
}

void    main()
{
    gl_Position = vec4(PosTimeVign.xy, 0.5, 1.0);
    mat4x4 lerpedEyeRot = EyeRotationStart * (1.0 - PosTimeVign.z) + EyeRotationEnd * PosTimeVign.z;
    fTexCoord0 = TimewarpTexCoord(TexCoord0, lerpedEyeRot);
    fTexCoord1 = TimewarpTexCoord(TexCoord1, lerpedEyeRot);
    fTexCoord2 = TimewarpTexCoord(TexCoord2, lerpedEyeRot);
    fVignette  = PosTimeVign.w;
}

#else

in vec2  fTexCoord0;
in vec2  fTexCoord1;
in vec2  fTexCoord2;
in float fVignette;

layout(location = 0) out vec4  OutColor;

void    main()
{
    // 3 samples for fixing chromatic aberrations
    float R = texture(Frame, fTexCoord0.xy).r;
    float G = texture(Frame, fTexCoord1.xy).g;
    float B = texture(Frame, fTexCoord2.xy).b;
    OutColor = fVignette * vec4(R, G, B, 1.0);
}

#endif
