#version 330 core
#auto_defines

uniform sampler2D   Scene;
uniform vec4        PosSize;    // (BH x, BH y, width, height)
uniform vec2        Scale;      // (BH scale x, BH scale y)
uniform vec3        RadDist;    // (BH radius, distance, 2*radius/distance)

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;
                     out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

                     in  vec2  TexCoord;
layout(location = 0) out vec4  OutColor;

void main()
{
	vec2  texCoord = TexCoord * PosSize.zw - PosSize.xy;
	float rad = length(texCoord * Scale.xy);
	float distort = max(RadDist.z / (rad * rad), 0.0);
	texCoord = texCoord * (1.0 - distort) + PosSize.xy;
    //texCoord = clamp(texCoord, vec2(0.0), PosSize.zw);
    OutColor = texture(Scene, texCoord);
	OutColor *= clamp((rad * RadDist.y - RadDist.x) / RadDist.x * 100.0, 0.0, 1.0); // antialiased event horizon
	//if (rad * RadDist.y < sqrt(RadDist.z) * RadDist.y) OutColor.g += 0.2; // check Einstein's radius
}

#endif
