#version 330 compatibility
precision highp float;

//attribute  vec3  gl_Vertex;           // Sprite center world position
//attribute  vec4  gl_Color;            // Sprite color, Sprite vertex Id
//attribute  vec2  gl_MultiTexCoord0;   // Radius, Bright

uniform sampler2D Texture;
uniform mat4x4    ProjectionMatrix;
uniform mat4x4    ModelViewMatrix;
uniform vec3      Param;    // StarFlareBeginBright, StarFlareBightCoef, StarFlareSize

#ifdef _VERTEX_

out vec2  TexCoord;
out vec3  Color;

void main()
{
    vec4  viewPos = ModelViewMatrix * gl_Vertex;
    float Dist2 = dot(viewPos.xyz, viewPos.xyz);
    float spriteSize = sqrt(Dist2) * pow(gl_MultiTexCoord0.x, 0.25) * Param.z;

    TexCoord.x = mod(floor(gl_Color.a * 4.0), 2.0);
    TexCoord.y = mod(floor(gl_Color.a * 2.0), 2.0);

    viewPos.xy += spriteSize * (TexCoord * 2.0 - 1.0);
    gl_Position = ProjectionMatrix * viewPos;

    Color = gl_Color.rgb * clamp(Param.y * (gl_MultiTexCoord0.x - Param.x), 0.0, 1.0);
}

#else

in vec2  TexCoord;
in vec3  Color;

void main()
{
/*
    vec2  point = TexCoord * 2.0 - 1.0;
    float r = length(point);
    float expon  = exp(-5.55 * r);
    float linear = 1.0 - r;

    float glow = 0.5 * expon;

    float streaks = (exp(-300.0 * abs(point.x)) + exp(-300.0 * abs(point.y)));
    streaks = 2.5 * pow(expon, 0.7) * streaks;

    float rays = 0.0;
    for (int i=0; i<64; ++i)
    {
        float fi = i * 3.1415926536 / 20.0;
        float sinf = sin(fi);
        float cosf = cos(fi);
        float px = point.x * sinf + point.y * cosf;
        //float py = point.x * cosf - point.y * sinf;
        rays += exp(-150.0 * abs(px));
    }
    rays = 0.1 * expon * rays;

    gl_FragColor.rgb = Color.rgb * (glow + streaks + rays);
*/
    gl_FragColor.rgb = Color.rgb * texture2D(Texture, TexCoord).rgb;
    gl_FragColor.a = 0;
}

#endif
