#version 330 core
#auto_defines

uniform sampler2D   BumpMap;
uniform vec4        FaceParams;  // (x0, y0, size, face)
uniform vec4        Scale;       // (xStep, yStep, height, offset)

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;
                     out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

                     in  vec2  TexCoord;
layout(location = 0) out vec3  OutPosition;
layout(location = 1) out vec3  OutNormal;
layout(location = 2) out vec2  OutTexCoord;
layout(location = 3) out vec3  OutTangent;

vec3    GetSurfacePoint(vec2 uv)
{
    vec2 spherical;
    spherical = uv * FaceParams.z + FaceParams.xy;
    vec3 p = normalize(vec3(spherical, 1.0));
    if (FaceParams.w == 0.0)      return vec3(-p.z, -p.y,  p.x);  // neg_x
    else if (FaceParams.w == 1.0) return vec3( p.z, -p.y, -p.x);  // pos_x
    else if (FaceParams.w == 2.0) return vec3( p.x, -p.z, -p.y);  // neg_y
    else if (FaceParams.w == 3.0) return vec3( p.x,  p.z,  p.y);  // pos_y
    else if (FaceParams.w == 4.0) return vec3(-p.x, -p.y, -p.z);  // neg_z
    else                          return vec3( p.x, -p.y,  p.z);  // pos_z
}

void main()
{
    vec2 dx = vec2(Scale.x, 0.0);
    vec2 dy = vec2(0.0, Scale.y);

    float h = (texture2D(BumpMap, TexCoord).r * Scale.y - Scale.z) + 1.0;
    vec3  p = GetSurfacePoint(TexCoord);
    vec3  t = p - GetSurfacePoint(TexCoord + dy);

    OutPosition.xyz = p * h;     // position
    OutNormal.xyz   = p;         // normal
    OutTexCoord.xy  = TexCoord;  // texture coordinates
    OutTangent.xyz  = normalize(t - dot(t, p) * p);  // tangent
}

#endif
