#version 330 compatibility
#auto_defines

//attribute float gl_Vertex;
//attribute float gl_MultiTexCoord0;

uniform sampler2D PrecomputedDensityFunc;
uniform mat4x4    Mvp;
uniform vec3      EyePos;
uniform vec4      BulgeColor;
uniform vec3      Reddening;

#ifdef _VERTEX_

out vec3  VertPos;

void main()
{
    gl_Position = Mvp * vec4(gl_Vertex.xyz, 1.0);
    VertPos = gl_Vertex.xyz;
}

#else

in vec3  VertPos;

void main()
{
    // Calculate parameters of the density function
    vec3   vertPos = normalize(VertPos);
    vec3   ray = EyePos + vertPos;                  // the ray from the pixel to the eye
    float  depth  = dot(vertPos, normalize(ray));   // half length of the part of the galaxy along the ray
    float  offset = sqrt(1.0 - depth*depth);        // the ray's offset from the galaxy's center
    depth = min(depth, 0.5*length(ray));            // cut the half length if the eye is inside the galaxy

    // Now get the visible brightness of the pixel
    float density = texture2D(PrecomputedDensityFunc, vec2(offset, depth)).r;

    // Calculate the final color
    gl_FragColor = density * density * BulgeColor;

    gl_FragColor.rgb *= Reddening;
    gl_FragColor = clamp(gl_FragColor, 0.0, 1.0);
}

#endif
