#version 330 compatibility
#auto_defines

//attribute float gl_Vertex;
//attribute float gl_MultiTexCoord0;

uniform sampler2D PrecomputedDensityFunc;
uniform mat4x4    Mvp;
uniform vec3      EyePos;
uniform vec4      BulgeColor;
uniform vec4      Clip;         // (Center, Radius^2)

#ifdef _VERTEX_

out vec3  VertPos;
out float fade;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);
    fade = clamp((Clip.w - Dist2) * 100.0, 0.0, 1.0);
    gl_Position = Mvp * vec4(gl_Vertex.xyz, 1.0);
    VertPos = gl_Vertex.xyz;
}

#else

in vec3  VertPos;
in float fade;

void main()
{
    // Calculate the parameters of the density function
    vec3   vertPos = normalize(VertPos);
    vec3   ray = EyePos + vertPos;                  // the ray from the pixel to the eye
    float  depth  = dot(vertPos, normalize(ray));   // half length of the part of the galaxy along the ray
    float  offset = sqrt(1.0 - depth*depth);        // the ray's offset from the galaxy's center
    depth = min(depth, 0.5*length(ray));            // cut the half length if the eye is inside the galaxy

    // Now get the visible brightness of the pixel
    float density = texture2D(PrecomputedDensityFunc, vec2(offset, depth)).r;

    // Calculate the final color
    gl_FragColor = density * density * fade * BulgeColor;
}

#endif
