#version 330 core
#auto_defines

uniform sampler2D    Base;
uniform sampler2D    Bloom;
uniform vec2         Param; // (BloomBright, Exposure)

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;

out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

in  vec2  TexCoord;

layout(location = 0) out vec4  OutColor;

void main()
{
    vec4 BaseColor  = texture(Base,  TexCoord);
    vec4 BloomColor = texture(Bloom, TexCoord);
    BloomColor.rgb = max(BloomColor.rgb - vec3(0.3), vec3(0.0));
    OutColor.rgb = BaseColor.rgb + BloomColor.rgb * Param.x;
    OutColor.rgb = 1.0 - exp(-Param.y * OutColor.rgb);
    OutColor.a = BaseColor.a;
}

#endif
