#version 330 core
#auto_defines

uniform sampler2D   guiTexture;
uniform vec4        Param;      // (scale.x, scale.y, LensAngle, CutOff)
uniform vec4        BackColor;
uniform vec3        Offset;
uniform mat3x3      Rotation;

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;

out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

in  vec2  TexCoord;

layout(location = 0) out vec4  OutColor;

void main()
{
    vec2  p = (TexCoord * 2.0 - 1.0) * Param.xy;
    float r = length(p);
    if (r > Param.w)
        OutColor = vec4(0.0);
    else
    {
        float Angle = Param.z * r;
        float sinr = sin(Angle) / r;
        vec3  ray = vec3(cos(Angle), sinr * p.y, sinr * p.x) + Offset;
		ray = Rotation * ray;
		if (ray.x > 0)
		{
			vec2 tc = (-ray.zy / ray.x + 1.0) * 0.5;
			OutColor = texture2D(guiTexture, tc);
		}
		else
			OutColor = vec4(0.0);
    }
}

#endif
