#version 330 core
#auto_defines

uniform sampler2D   Scene;
uniform vec4        PosSize;    // (center x, center y, width, height)
uniform vec4        Scale;      // (BH scale x, BH scale y)
uniform vec4        RadDist;    // (BH radius, distance, 2*radius/distance, 2*radius/distance/strength)
uniform vec4        Axis;       // (Gravity xy, Antigravity xy)

#ifdef _VERTEX_

layout(location = 0) in  vec4  VertexPos;
layout(location = 1) in  vec4  VertexTexCoord;
                     out vec2  TexCoord;

void main()
{
    gl_Position = VertexPos;
    TexCoord = VertexTexCoord.xy;
}

#else

                     in  vec2  TexCoord;
layout(location = 0) out vec4  OutColor;

void main()
{
    vec2  texCoord =   TexCoord            * PosSize.zw - PosSize.xy;
    vec2  texCoordG = (TexCoord + Axis.xy) * PosSize.zw - PosSize.xy;
    vec2  texCoordA = (TexCoord + Axis.zw) * PosSize.zw - PosSize.xy;
	float radG = length(texCoordG * Scale.xy) + RadDist.w;
	float radA = length(texCoordA * Scale.xy) + RadDist.w;
	float distort = max(RadDist.z / (radG * radG), 0.0) - max(RadDist.z / (radA * radA), 0.0);
	texCoord = texCoord * (1.0 - distort) + PosSize.xy;
    //texCoord = clamp(texCoord, vec2(0.0), PosSize.zw);
    OutColor = texture(Scene, texCoord);
}

#endif
