#include "tg_common.glsl"

#ifdef _FRAGMENT_

//-----------------------------------------------------------------------------

float   Fbm2(vec3 point)
{
    float summ = 0.0;
	float ampl = 1.0;
    for (int i=0; i<cloudsOctaves; ++i)
    {
        summ  += Noise(point) * ampl;
		ampl  *= 0.333;
        point *= 3.1416;
    }
    return summ;
}

//-----------------------------------------------------------------------------

vec3    Fbm3D2(vec3 point)
{
    vec3  summ = vec3(0.0);
	float ampl = 1.0;
    for (int i=0; i<cloudsOctaves; ++i)
    {
        summ  += NoiseVec3(point) * ampl;
		ampl  *= 0.333;
        point *= 3.1416;
    }
    return summ;
}

//-----------------------------------------------------------------------------

vec4    Cell3NoiseVecT2(vec3 p, float jitter)
{
    vec3  cell = floor(p);
	float f1 = 1000;
    float f2 = 1000;
    vec3  pos1, pos2;
    vec3  d;
    for (d.z=-1.0; d.z<2.0; d.z+=1.0)
    {
        for (d.y=-1.0; d.y<2.0; d.y+=1.0)
        {
            for (d.x=-1.0; d.x<2.0; d.x+=1.0)
            {
				vec3 testcell = cell + d;
                vec3 pos = testcell + jitter * NoiseNearestUVec4(testcell / NOISE_TEX_3D_SIZE).xyz;
                //vec3 pos = testcell + jitter * permute3(testcell).xyz;
				vec3 offs = pos - p;
				float dist = dot(offs, offs);
				if (dist < f1)
                {
                    f2 = f1;    pos2 = pos1;
                    f1 = dist;  pos1 = pos;
                }
                else if (dist < f2)
                {
                    f2 = dist;  pos2 = pos;
				}
    		}
		}
    }

    pos1 = normalize(pos1 + cell + vec3(0.5));
    pos1 = pos1 * 2.0 - vec3(1.0);
    return vec4(pos1, sqrt(f1));
}

//-----------------------------------------------------------------------------

vec4    Cell3NoiseVecT1(vec3 p, float jitter)
{
    vec3  cell = floor(p);
    vec3  offs = p - cell;
    vec3  pos;
    vec3  point = vec3(0.0);
    vec3  rnd;
    vec3  d;
    float dist;
    float distMin = 1.0e38;
    for (d.z=-1.0; d.z<2.0; d.z+=1.0)
    {
        for (d.y=-1.0; d.y<2.0; d.y+=1.0)
        {
            for (d.x=-1.0; d.x<2.0; d.x+=1.0)
            {
                rnd = NoiseNearestUVec4((cell + d) / NOISE_TEX_3D_SIZE).xyz * jitter + d;
                pos = rnd - offs;
                dist = dot(pos, pos);
                if (distMin > dist)
                {
                    distMin = dist;
                    point = rnd;
                }
            }
        }
    }
    //point = normalize(point + cell + vec3(0.5));
    //return vec4(point * 2.0 - vec3(1.0), sqrt(distMin));
    point = normalize(point + cell + vec3(0.5));
    point = point * 2.0 - vec3(1.0);
    return vec4(point, sqrt(distMin));
}

//-----------------------------------------------------------------------------
// Cellular noise, returning F1 and F2 in a vec2.
// Speeded up by using 2x2x2 search window instead of 3x3x3,
// at the expense of some pattern artifacts.
// F2 is often wrong and has sharp discontinuities.
// If you need a good F2, use the slower 3x3x3 version.
// default jitter = 0.8 - smaller jitter gives less errors in F2

vec4 Cell2NoiseVecT0(vec3 P, float jitter)
{
#define K 0.142857142857 // 1/7
#define Ko 0.428571428571 // 1/2-K/2
#define K2 0.020408163265306 // 1/(7*7)
#define Kz 0.166666666667 // 1/6
#define Kzo 0.416666666667 // 1/2-1/6*2

    vec3 Pi = mod(floor(P), 289.0);
 	vec3 Pf = fract(P);
	vec4 Pfx = Pf.x + vec4(0.0, -1.0, 0.0, -1.0);
	vec4 Pfy = Pf.y + vec4(0.0, 0.0, -1.0, -1.0);
	vec4 p = permute(Pi.x + vec4(0.0, 1.0, 0.0, 1.0));
	p = permute(p + Pi.y + vec4(0.0, 0.0, 1.0, 1.0));
	vec4 p1 = permute(p + Pi.z); // z+0
	vec4 p2 = permute(p + Pi.z + vec4(1.0)); // z+1
	vec4 ox1 = fract(p1*K) - Ko;
	vec4 oy1 = mod(floor(p1*K), 7.0)*K - Ko;
	vec4 oz1 = floor(p1*K2)*Kz - Kzo; // p1 < 289 guaranteed
	vec4 ox2 = fract(p2*K) - Ko;
	vec4 oy2 = mod(floor(p2*K), 7.0)*K - Ko;
	vec4 oz2 = floor(p2*K2)*Kz - Kzo;
	vec4 dx1 = Pfx  + jitter*ox1;
	vec4 dy1 = Pfy  + jitter*oy1;
	vec4 dz1 = Pf.z + jitter*oz1;
	vec4 dx2 = Pfx  + jitter*ox2;
	vec4 dy2 = Pfy  + jitter*oy2;
	vec4 dz2 = Pf.z + jitter*oz2 - 1.0;
	vec4 d1 = dx1 * dx1 + dy1 * dy1 + dz1 * dz1; // z+0
	vec4 d2 = dx2 * dx2 + dy2 * dy2 + dz2 * dz2; // z+1

    vec3  point = Pf;
    float minDist = 1.0e38;
    if (minDist > d1.x) { minDist = d1.x; point = Pfx.xyz; }
    if (minDist > d1.y) { minDist = d1.y; point = Pfy.xyz; }
    if (minDist > d1.z) { minDist = d1.z; point = Pf;      }
    if (minDist > d2.x) { minDist = d2.x; point = Pfx.xyz; }
    if (minDist > d2.y) { minDist = d2.y; point = Pfy.xyz; }
    if (minDist > d2.z) { minDist = d2.z; point = Pf;      }
	return vec4(point.xyz, sqrt(minDist));

	d1 = min(d1, d2);
	d1.xy = min(d1.xy, d1.wz);
	d1.x = min(d1.x, d1.y);
	return vec4(1.0, 1.0, 1.0, sqrt(d1.x));
}

//-----------------------------------------------------------------------------

void main()
{
    float height = GetSurfaceHeight();
    OutColor = GetCloudsColor(height) * height;
/*
    vec3  point  = GetSurfacePoint();

    float zone = cos(point.y * twistZones);
    float ang = zone * twistMagn;
    vec3  twistedPoint = point;
	float coverage = cloudsCoverage;
    float weight = 1.0;

    // Compute the cyclons
    vec4  cycloneCenter0 = Cell2NoiseVecT0((point + Randomize) * cycloneFreq * 10.0, 0.8);
    vec4  cycloneCenter1 = Cell3NoiseVecT1((point + Randomize) * cycloneFreq * 10.0, 0.8);
    vec4  cycloneCenter2 = Cell3NoiseVecT2((point + Randomize) * cycloneFreq * 10.0, 0.8);

    //vec4  cycloneCenter = (point.y <= 10.0) ? cycloneCenter1 : cycloneCenter2;
    vec4  cycloneCenter = cycloneCenter1;

    OutColor.rgb = cycloneCenter.xyz * cycloneCenter.w;
    OutColor.a = 1.0;
/*
    float cycloneRadius = length(cycloneCenter.xyz - point) / cycloneSqrtDensity;
    float cycloneAmpl   = -tidalLock * cycloneMagn * sign(cycloneCenter.y);

    if (cycloneRadius < 1.0)
    {
        float dist = 1.0 - cycloneRadius;
        float fi = mix(log(cycloneRadius), pow(dist, 3.0), cycloneRadius);
        twistedPoint = Rotate(cycloneAmpl*fi, cycloneCenter.xyz, point);
        weight = saturate(1.0 - cycloneRadius / 0.05);
        weight = (1.0 - weight * weight) * (1.0 + dist);
		coverage = mix(coverage, 1.0, dist);
    }

    // Compute the Coriolis effect
    float sina = sin(ang);
    float cosa = cos(ang);
    twistedPoint = vec3(cosa*twistedPoint.x-sina*twistedPoint.z, twistedPoint.y, sina*twistedPoint.x+cosa*twistedPoint.z);
    twistedPoint = twistedPoint * cloudsFreq + Randomize;

    // Compute the flow-like distortion
	vec3 p = twistedPoint * cloudsFreq * 6.37;
	vec3 q = p + Fbm3D2(p);
	vec3 r = p + Fbm3D2(q);
	float f = Fbm2(r) * 0.7 + coverage - 0.3;
	float global = saturate(f) * weight;

    OutColor = vec4(global);
/**/
}

//-----------------------------------------------------------------------------

#endif
