#include "tg_common.glsl"

#ifdef _FRAGMENT_

//-----------------------------------------------------------------------------

float   Fbm2(vec3 point)
{
    float summ = 0.0;
	float ampl = 1.0;
    for (int i=0; i<cloudsOctaves; ++i)
    {
        summ  += Noise(point) * ampl;
		ampl  *= 0.333;
        point *= 3.1416;
    }
    return summ;
}

//-----------------------------------------------------------------------------

vec3    Fbm3D2(vec3 point)
{
    vec3  summ = vec3(0.0);
	float ampl = 1.0;
    for (int i=0; i<cloudsOctaves; ++i)
    {
        summ  += NoiseVec3(point) * ampl;
		ampl  *= 0.333;
        point *= 3.1416;
    }
    return summ;
}

//-----------------------------------------------------------------------------
/*
void    CycloneNoise(vec3 point, inout vec3 twistedPoint, inout float weight, inout float coverage)
{
    vec3  p = point;
    float freq = cycloneFreq               *  1.0;
    float dens = 1.0 / cycloneSqrtDensity  *  1.0;

    for (int i=0; i<3; i++)
    {
        vec3  cycloneCenter = Cell3NoiseVec(p * freq, 0.6).xyz;
        float cycloneRadius = length(cycloneCenter - p) * dens;
        float cycloneAmpl   = -tidalLock * cycloneMagn * sign(cycloneCenter.y);
        if (cycloneRadius < 1.0)
        {
            float dist = 1.0 - cycloneRadius;
            float fi = mix(log(cycloneRadius), pow(dist, 3.0), cycloneRadius);
            twistedPoint = Rotate(cycloneAmpl*fi, cycloneCenter, point);
            float w = saturate(1.0 - cycloneRadius / 0.05);
            weight += (1.0 - w * w) * (1.0 + dist);
	        coverage = mix(coverage, 1.0, dist);
        }
        p += Randomize;
        freq *= 0.5;
        dens *= 2.0;
    }
    weight = saturate(weight);
}
*/
//-----------------------------------------------------------------------------

float   HeightMapClouds(vec3 point)
{
    float zone = cos(point.y * twistZones);
    float ang = zone * twistMagn;
    vec3  twistedPoint = point;
	float coverage = cloudsCoverage;
    float weight = 1.0;

#if 1
    // Compute the cyclons
    vec3  cycloneCenter = (tidalLock > 0.0) ? vec3(0.0, 1.0, 0.0) : Cell2NoiseVec(point * cycloneFreq, 0.6).xyz;
    float cycloneRadius = length(cycloneCenter - point) / cycloneSqrtDensity;
    float cycloneAmpl   = -tidalLock * cycloneMagn * sign(cycloneCenter.y);
    if (cycloneRadius < 1.0)
    {
        float dist = 1.0 - cycloneRadius;
        float fi = mix(log(cycloneRadius), pow(dist, 3.0), cycloneRadius);
        twistedPoint = Rotate(cycloneAmpl*fi, cycloneCenter, point);
        weight = saturate(1.0 - cycloneRadius / 0.05);
        weight = (1.0 - weight * weight) * (1.0 + dist);
		coverage = mix(coverage, 1.0, dist);
    }
#else
    // Compute the cyclons
    if (tidalLock > 0.0)
    {
        vec3  cycloneCenter = vec3(0.0, 1.0, 0.0);
        float cycloneRadius = length(cycloneCenter - point) / cycloneSqrtDensity;
        float cycloneAmpl   = -tidalLock * cycloneMagn * sign(cycloneCenter.y);
        if (cycloneRadius < 1.0)
        {
            float dist = 1.0 - cycloneRadius;
            float fi = mix(log(cycloneRadius), pow(dist, 3.0), cycloneRadius);
            twistedPoint = Rotate(cycloneAmpl*fi, cycloneCenter, point);
            weight = saturate(1.0 - cycloneRadius / 0.05);
            weight = (1.0 - weight * weight) * (1.0 + dist);
	        coverage = mix(coverage, 1.0, dist);
        }
    }
    else
        CycloneNoise(point, twistedPoint, weight, coverage);
#endif

    // Compute the Coriolis effect
    float sina = sin(ang);
    float cosa = cos(ang);
    twistedPoint = vec3(cosa*twistedPoint.x-sina*twistedPoint.z, twistedPoint.y, sina*twistedPoint.x+cosa*twistedPoint.z);
    twistedPoint = twistedPoint * cloudsFreq + Randomize;

    // Compute the flow-like distortion
	vec3 p = twistedPoint * cloudsFreq * 6.37;
	vec3 q = p + Fbm3D2(p);
	vec3 r = p + Fbm3D2(q);
	float f = Fbm2(r) * 0.7 + coverage - 0.3;
	float global = saturate(f) * weight;

    // Compute turbilence features
    //noiseOctaves = cloudsOctaves;
    //float turbulence = (Fbm(point * 100.0 * cloudsFreq + Randomize) + 1.5);// * smoothstep(0.0, 0.05, global);

    return global;
}

//-----------------------------------------------------------------------------

void main()
{
    vec3  point  = GetSurfacePoint();
    float height = HeightMapClouds(point);
    OutColor = vec4(height);
}

//-----------------------------------------------------------------------------

#endif
