#include "tg_common.glsl"

#ifdef _FRAGMENT_

//-----------------------------------------------------------------------------

vec4  ColorMapSelena(vec3 point, float height, float slope)
{
    // Assign a material
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    float mat, dist;

    if (tidalLock <= 0.0)
        mat = height;
    else
    {
        float latitude = 1.0 - point.x;
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        mat = mix(climateTropic, climatePole, latitude);
    }

    // Color texture distortion
    noiseOctaves = 15.0;
    dist = 1.5 * floor(2.0 * DistFbm(point * 0.002 * colorDistFreq, 2.0));
    mat += colorDistMagn * dist;

    // Color texture variation
/*
    noiseOctaves = 8.0;
    //float vary = 4.0 * abs(Fbm(point * 15 * colorDistFreq));
    vec3 p = point * 10.0 * colorDistFreq;
    p += Fbm3D(p);
    float vary = Cell2Noise(p);
*/
    noiseOctaves = 5;
    vec3 p = point * colorDistFreq * 2.3;
    p += Fbm3D(p * 0.5) * 1.2;
    float vary = saturate((Fbm(p) + 0.7) * 0.7);

    // Shield volcano lava
    if (volcanoOctaves > 0)
    {
        // Global volcano activity mask
        noiseOctaves = 3;
        float volcActivity = saturate((Fbm(point * 1.37 + Randomize) - 1.0 + volcanoActivity) * 5.0);
        // Lava in volcano caldera and flows
	    float lavaMask = VolcanoGlowNoise(point) * volcActivity;
		// Model lava as rocks texture
		mat   = mix(mat,   0.0, lavaMask);
		slope = mix(slope, 0.0, lavaMask);
    }

    Surface surf = GetSurfaceColor(saturate(mat), slope, vary);

    // Ice cracks
    vec4 colorMult = vec4(1.0);
    if (cracksOctaves > 0.0)
    {
        // Rim height distortion
        noiseH          = 0.5;
        noiseLacunarity = 2.218281828459;
        noiseOffset     = 0.8;
        noiseOctaves    = 4.0;
        float dunes = 0.7 * max(Fbm(point * dunesFreq) + 0.7, 0.0);
        noiseOctaves    = 6.0;
        float crack = CrackColorNoise(point);// * (1.0 + 0.3 * dunes);
        colorMult = mix(vec4(0.67, 0.58, 0.36, 0.00), colorMult, crack);
        //slope = 1.0 - smoothstep(0.0, 0.05, crack);
    }

    surf.color *= colorMult;

    // Albedo variations
    noiseOctaves = 4;
    vec3 distort = Fbm3D((point + Randomize) * 0.07) * 0.5;
    noiseOctaves = 8;
    vary = Fbm((point + distort) * 0.78);
	surf.color *= 1.0 - vary;
/*
    // Albedo variations
    // This code is broken on some AMD cards
    p = point * mainFreq + Randomize;
    noiseOctaves = 5;
    vec3 distort = 0.35 * Fbm3D(p * 2.37);
    noiseOctaves = 4;
    p += distort + 0.005 * (1.0 - abs(Fbm3D(p * 132.3)));
    surf.color.rgb = surf.color.rgb - 1.0 * saturate(1.0 - Cell3Noise(0.6 * p));
/**/

    // Make driven hemisphere darker
    if (drivenDarkening != 0.0)
    {
        noiseOctaves = 3;
        float z = -point.z * sign(drivenDarkening);
        z += 0.2 * Fbm(point * 1.63);
        z = saturate(1.0 - z);
        z *= z;
        surf.color.rgb *= mix(1.0 - abs(drivenDarkening), 1.0, z);
    }
        
    // Rayed craters
    float craterRayedSqrtDensity = craterSqrtDensity * sqrt(craterRayedFactor);
    if (craterRayedSqrtDensity > 0.05)
    {
        noiseH           = 0.5;
        noiseLacunarity  = 2.218281828459;
        noiseOffset      = 0.8;
        noiseOctaves     = 3;  // Craters roundness distortion
        craterDistortion = 1.0;
        craterRoundDist  = 0.03;
        heightFloor      = -0.5;
        heightPeak       = 0.6;
        heightRim        = 1.0;
        float craterRayedOctaves = floor(craterOctaves * craterRayedFactor);
        float crater = RayedCraterColorNoise(point, craterFreq, craterRayedSqrtDensity, craterRayedOctaves);
        surf.color.rgb += vec3(0.5 * crater);
    }

    return surf.color;
}

//-----------------------------------------------------------------------------

void main()
{
    vec3  point = GetSurfacePoint();
    float height, slope;
    GetSurfaceHeightAndSlope(height, slope);
    OutColor = ColorMapSelena(point, height, slope);
}

//-----------------------------------------------------------------------------

#endif
