#include "tg_common.glsl"

#ifdef _FRAGMENT_

//-----------------------------------------------------------------------------

vec4  GlowMapTerra(vec3 point, float height, float slope)
{
    // Assign a climate type
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    noiseH          = 0.5;
    noiseLacunarity = 2.218281828459;
    noiseOffset     = 0.8;
    float climate, latitude, dist;
    if (tidalLock <= 0.0)
    {
        latitude = abs(point.y);
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        latitude = saturate(latitude);
        if (latitude < latTropic - tropicWidth)
            climate = mix(climateTropic, climateEquator, (latTropic - tropicWidth - latitude) / latTropic);
        else if (latitude > latTropic + tropicWidth)
            climate = mix(climateTropic, climatePole, (latitude - latTropic - tropicWidth) / (1.0 - latTropic));
        else
            climate = climateTropic;
    }
    else
    {
        latitude = 1.0 - point.x;
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        climate = mix(climateTropic, climatePole, saturate(latitude));
    }

    // Change climate with elevation
    float montHeight = saturate((height - seaLevel) / (snowLevel - seaLevel));
    climate = min(climate + heightTempGrad * montHeight, climatePole);

    // Ice caps
    float iceCap = saturate((latitude / latIceCaps - 1.0) * 50.0);
    climate = mix(climate, climatePole, iceCap);

/*
    // Thermal glow variations
    noiseOctaves = 5;
    vec3 p = point * 600.0 + Randomize;
    dist = 10.0 * colorDistMagn * Fbm3D(p * 0.2);
    //vec2 cell = Cell3Noise2(p + dist);
    ////float varyTemp = 0.5 * smoothstep(0.0, 0.4, cell.y - cell.x);
    ////float varyTemp = 0.5 * sqrt(abs(cell.y - cell.x));
    //float varyTemp = 1.0 - 5.0 * smoothstep(0.1, 1.0, sqrt(abs(cell.y - cell.x)));
    //float flow  = saturate(varyTemp * 0.2 * lavaCoverage);
    noiseOctaves = 8;
	float varyTemp = abs(Fbm(p + dist));
	// Thermal emission temperature (in thousand Kelvins)
	float surfTemp = 0.0;
*/
	// Thermal emission temperature (in thousand Kelvins)
    vec3  p = point * 600.0 + Randomize;
    noiseOctaves = 5;
    dist = 10.0 * colorDistMagn * Fbm(p * 0.2);
    /*vec2 cell = Cell3Noise2(p + dist);
    //float varyTemp = 0.5 * smoothstep(0.0, 0.4, cell.y - cell.x);
    //float varyTemp = 0.5 * sqrt(abs(cell.y - cell.x));
    float varyTemp = 1.0 - 5.0 * smoothstep(0.1, 1.0, sqrt(abs(cell.y - cell.x)));
    float flow  = saturate(varyTemp * 0.2 * lavaCoverage);*/
    noiseOctaves = 3;
	float globTemp = 0.95 - abs(Fbm((p + dist) * 0.01)) * 0.08;
    noiseOctaves = 8;
	float varyTemp = abs(Fbm(p + dist));

	float surfTemp = surfTemperature *
		(globTemp + varyTemp * 0.08) *
		saturate(2.0 * (lavaCoverage * 0.4 + 0.4 - 0.8 * height)) *
		saturate((lavaCoverage - 0.01) * 25.0) *
		saturate((0.875 - climate) * 50.0);

    // Shield volcano lava
    if (volcanoOctaves > 0)
    {
        // Global volcano activity mask
        noiseOctaves = 3;
        float volcActivity = saturate((Fbm(point * 1.37 + Randomize) - 1.0 + volcanoActivity) * 5.0);
        //float volcActivity = saturate((Fbm(point * 1.37 + Randomize) - 1.0 + 0.5) * 5.0);
        // Lava in the volcano caldera and lava flows
		float volcTemp = VolcanoGlowNoise(point) * (0.75 + 0.25 * varyTemp) * volcActivity * volcanoTemp;
		//float volcTemp = (0.75 + 0.25 * varyTemp) * volcActivity * volcanoTemp;
	    surfTemp = max(surfTemp, volcTemp);
    }

    // Io-like volcanoes
    //float volcIo = saturate(abs(Fbm(point * 6.3 + Randomize) * 1.4)); // * 1.4 * lavaCoverage));

	//float r = log(surfTemp) * 0.188 + 0.1316;
	//return vec4(r, 0.0, 0.0, 1.0);

	vec4  outColor = vec4(0.0);

	//surfTemp *= 3.0;
	//float thermFade = smoothstep(0.5, 0.8, surfTemp);
	//if (surfTemp > 0.5)
	{
		outColor.rgb = UnitToColor24(log(surfTemp) * 0.188 + 0.1316);// * thermFade;
		outColor.a = 1.0;
	}
	/*else
	{
		noiseOctaves = 5;
		p = point * colorDistFreq + Randomize;
		dist = colorDistMagn * Fbm3D(p * 0.2);
		vec2  cell = Cell3Noise2(p + dist);
		float street = smoothstep(0.1, 1.0, sqrt(abs(cell.y - cell.x)));
		float city = saturate((Fbm(point * 1.28 + Randomize) - 1.0 + 0.3) * 5.0) * (1.0 - street);
		float cityFade = smoothstep(0.1, 0.3, city);

		if (city > 0.1)
		{
			outColor.rgb = vec3(1.0, 0.7, 0.2) * city * cityFade;
			outColor.a = 0.0;
		}
		else
		{
			noiseOctaves = 5;
			p = point * colorDistFreq * 2.0 + Randomize;
			dist = colorDistMagn * Fbm3D(p * 0.2);
			cell = Cell3Noise2(p + dist);
			street = smoothstep(0.1, 1.0, sqrt(abs(cell.y - cell.x)));
			float perm = saturate((Fbm(point * 1.49 + Randomize) - 1.0 + 0.3) * 5.0) * (1.0 - street);
			float permFade = smoothstep(0.1, 0.3, perm);

			outColor.rgb = vec3(0.3, 1.0, 0.1) * perm * (1.0 - cityFade) * (1.0 - thermFade) * permFade;
			outColor.a = 0.5;
		}
	}*/

	return outColor;

	//vec4 outColor;
	//outColor.rgb = UnitToColor24(surfTemp * 0.1);
	//outColor.a = 1.0;
	//return outColor;
}

//-----------------------------------------------------------------------------

void main()
{
    vec3  point = GetSurfacePoint();
    float height, slope;
    GetSurfaceHeightAndSlope(height, slope);
    OutColor = GlowMapTerra(point, height, slope);
}

//-----------------------------------------------------------------------------

#endif
