#include "tg_common.glsl"

#ifdef _FRAGMENT_

//-----------------------------------------------------------------------------

float   HeightMapTerra(vec3 point)
{
    // Global landscape
    vec3  p = point * mainFreq + Randomize;
    noiseOctaves = 5;
    vec3  distort = 0.35 * Fbm3D(p * 2.37);
    noiseOctaves = 4;
    distort += 0.005 * (1.0 - abs(Fbm3D(p * 132.3)));
    float global = 1.0 - Cell3Noise(p + distort);

    // Venus-like structure
    float venus = 0.0;
    if (venusMagn > 0.05)
    {
        noiseOctaves = 4;
        distort = Fbm3D(point * 0.3) * 1.5;
        noiseOctaves = 6;
        venus = Fbm((point + distort) * venusFreq) * venusMagn;
    }

    global = (global + venus - seaLevel) * 0.5 + seaLevel;
    float shore = saturate(70.0 * (global - seaLevel));

    vec4  col;
    noiseOctaves = 6;
    p = p * 2.3 + 13.5 * Fbm3D(p * 0.06);
    vec2  cell = Cell3Noise2Color(p, col);
    float biome = col.r;
    float biomeScale = saturate(2.0 * (pow(abs(cell.y - cell.x), 0.7) - 0.05));
    float terrace = col.g;
    float terraceLayers = max(col.b * 10.0 + 3.0, 3.0);
    terraceLayers += Fbm(p * 5.41);

    float montRage = saturate(DistNoise(point * 22.6 + Randomize, 2.5) + 0.5);
    montRage *= montRage;
    float montBiomeScale = min(pow(2.2 * biomeScale, 2.5), 1.0) * montRage;

    float inv2montesSpiky = 1.0 /(montesSpiky*montesSpiky);
    float height = 0.0;
    float dist;

    if (biome < dunesFraction)
    {
        // Dunes
        noiseOctaves = 2.0;
        dist = dunesFreq + Fbm(p * 1.21);
        height = max(Fbm(p * dist * 0.3) + 0.7, 0.0);
        height = biomeScale * dunesMagn * (height + DunesNoise(point, 3));
    }
    else if (biome < hillsFraction)
    {
        // "Eroded" hills
        noiseOctaves = 10.0;
        noiseH       = 1.0;
        noiseOffset  = 1.0;
        height = biomeScale * hillsMagn * (1.5 - RidgedMultifractal(point * hillsFreq + Randomize, 2.0));
    }
    else if (biome < hills2Fraction)
    {
        // "Eroded" hills 2
        noiseOctaves = 10.0;
        noiseLacunarity = 2.0;
        height = biomeScale * hillsMagn * JordanTurbulence(point * hillsFreq + Randomize, 0.8, 0.5, 0.6, 0.35, 1.0, 0.8, 1.0);
    }
    else if (biome < canyonsFraction)
    {
        // Canyons
        noiseOctaves = 5.0;
        noiseH       = 0.9;
        noiseLacunarity = 4.0;
        noiseOffset  = montesSpiky;
        height = -canyonsMagn * montRage * RidgedMultifractalErodedDetail(point * 4.0 * canyonsFreq * inv2montesSpiky + Randomize, 2.0, erosion, montBiomeScale);

        //if (terrace < terraceProb)
        {
            terraceLayers *= 5.0;
            float h = height * terraceLayers;
            height = (floor(h) + smoothstep(0.1, 0.9, fract(h))) / terraceLayers;
        }
    }
    else
    {
        // Mountains
        noiseOctaves = 10.0;
        noiseH       = 1.0;
        noiseLacunarity = 4.0;
        noiseOffset  = montesSpiky;
        height = montesMagn * montRage * RidgedMultifractalErodedDetail(point * montesFreq * inv2montesSpiky + Randomize, 2.0, erosion, montBiomeScale);

        if (terrace < terraceProb)
        {
            float h = height * terraceLayers;
            height = (floor(h) + smoothstep(0.1, 0.9, fract(h))) / terraceLayers;
        }
    }

    height *= shore;

    // Mare
    float mare = global;
    float mareFloor = global;
    float mareSuppress = 1.0;
    if (mareSqrtDensity > 0.05)
    {
        //noiseOctaves = 2;
        //mareFloor = 0.6 * (1.0 - Cell3Noise(0.3*p));
        noiseH           = 0.5;
        noiseLacunarity  = 2.218281828459;
        noiseOffset      = 0.8;
        craterDistortion = 1.0;
        noiseOctaves     = 6.0;  // Mare roundness distortion
        mare = MareNoise(point, global, 0.0, mareSuppress);
    }

    // Ice cracks
    if (cracksOctaves > 0.0)
    {
        // Rim height distortion
        noiseH          = 0.5;
        noiseLacunarity = 2.218281828459;
        noiseOffset     = 0.8;
        noiseOctaves    = 4.0;
        float dunes = 2 * cracksMagn * (0.2 + dunesMagn * max(Fbm(point * dunesFreq) + 0.7, 0.0));
        noiseOctaves    = 6.0;  // Cracks roundness distortion
        height += CrackNoise(point, dunes);
    }

    // Craters
    float crater = 0.0;
    if (craterSqrtDensity > 0.05)
    {
        noiseOctaves = 3;  // Craters roundness distortion
        craterDistortion = 1.0;
        craterRoundDist  = 0.03;
        heightFloor = -0.1;
        heightPeak  = 0.6;
        heightRim   = 1.0;
        crater = CraterNoise(point, 0.5 * craterMagn, craterFreq, craterSqrtDensity, craterOctaves);
        noiseOctaves = 10.0;
        noiseH       = 1.0;
        noiseOffset  = 1.0;
        crater = RidgedMultifractalErodedDetail(point * 0.3 * montesFreq + Randomize, 2.0, erosion, 0.25 * crater);
    }

    height += mare + crater;

    // Pseudo rivers
    if (riversOctaves > 0)
    {
        noiseOctaves     = riversOctaves;
        noiseLacunarity  = 2.218281828459;
        noiseH           = 0.5;
        noiseOffset      = 0.8;
        p = point * mainFreq + Randomize;
        distort  = 0.035 * Fbm3D(p * riversSin * 5.0);
        distort += 0.350 * Fbm3D(p * riversSin);
        cell = Cell3Noise2(riversFreq * p + distort);
        float pseudoRivers = 1.0 - saturate(abs(cell.y - cell.x) * riversMagn);
        pseudoRivers = smoothstep(0.0, 1.0, pseudoRivers);
        pseudoRivers *= 1.0 - smoothstep(0.06, 0.10, global - seaLevel); // disable rivers inside continents
        pseudoRivers *= 1.0 - smoothstep(0.00, 0.01, seaLevel - height); // disable rivers inside oceans
        height = mix(height, seaLevel-0.02, pseudoRivers);
    }

    // Shield volcano
    if (volcanoOctaves > 0)
        height = VolcanoNoise(point, global, height);

    // Assign a climate type
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    noiseH          = 0.5;
    noiseLacunarity = 2.218281828459;
    noiseOffset     = 0.8;
    float latitude;
    if (tidalLock <= 0.0)
    {
        latitude = abs(point.y);
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        latitude = saturate(latitude);
    }
    else
    {
        latitude = 1.0 - point.x;
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
    }
/*
    // Ice caps
    float iceCap;
    if (cloudsStyle == 1.0)
    {
        iceCap = saturate(15.0 * (latitude - latIceCaps));
        height = height * 0.1 + icecapHeight * iceCap;
    }
    else
    {
        iceCap = saturate(3.0 * (latitude - latIceCaps));
        height += icecapHeight * sqrt(iceCap);
    }
/*
    // Ice caps
    float iceCap = saturate((latitude / latIceCaps - 1.0) * 50.0);
    height = max(height, icecapHeight * iceCap);
/**/
    // Ice caps;
    // cloudsStyle = 0.1 for oceania, 1.0 for other planets
    float iceCap = saturate((latitude / latIceCaps - 1.0) * 50.0 * cloudsStyle);
    height = height * cloudsStyle + icecapHeight * smoothstep(0.0, 1.0, iceCap);
/**/
    //cell = Cell3Noise2(point * 103.7 + Randomize);
    //float glacier = smoothstep(0.1, 0.5, (abs(cell.y - cell.x)));

    //noiseOctaves = 5.0;
    //noiseH       = 0.9;
    //noiseLacunarity = 4.0;
    //noiseOffset  = montesSpiky;
    //float glacier;
    //glacier = RidgedMultifractalDetail(point * canyonsFreq * inv2montesSpiky + Randomize, 2.0, iceCap);

    //height = 1.0 - 0.03 * icecapHeight * glacier;
    //height = max(height, iceCap * glacier);

    return height;
}

//-----------------------------------------------------------------------------

void main()
{
    vec3  point = GetSurfacePoint();
    float height = HeightMapTerra(point);
    OutColor = vec4(height);
}

//-----------------------------------------------------------------------------

#endif
